#!/usr/bin/perl
## ######### PROJECT NAME : ##########
##
## file_under_control.pl
##
## ######### PROJECT DESCRIPTION : ###
##
##
##
## ###################################
##
## Made by Pujos Sylvain et Dayan Romain
##
## Started on  Jan 18 11:01:00 2005 Pujos Sylvain et Dayan Romain
##
## ###################################
## Version = O.9


use	strict;
use	Data::Dumper;
use	Getopt::Long;		# paramtres ligne de commande

use	POSIX('strftime');

#load some modules.
use	File::Find;
use	SGI::FAM;

my $DEBUG	= 0;
my $ALL		= 0;
my $CHANGE	= 0;
my $CREATE	= 1;
my $DELETE	= 1;

# Definition Samba
my $smb_status	= "/usr/bin/smbstatus";
my $smb_conf	= "/etc/samba/smb.conf";

# Definition log
my $log_file	= "/var/log/storiq/file_under_control.log";

# Definition du repertoire racine
my $directory	= "/mnt/raid/";

#start a fam object
my $fam		= new SGI::FAM;
my $event;
my $dir;
my $file;
my $filepath;

# # Recuperation des arguments
# getCLOptions();
# 
# # Recuperation du repertoire a surveiller
# if (scalar(@ARGV)) {
# 	$directory = shift @ARGV;
# }
$directory = SGI::FAM::abspath($directory);
print_error( "$directory n'est pas un repertoire" ) unless ( -d $directory );

# # Si Debug, on logue tout
# if ( $DEBUG or $ALL ) {
# 	$CHANGE	= "1";
# 	$CREATE	= "1";
# 	$DELETE	= "1";
# }

# Ajout des repertoires a surveiller
add_directory_to_fam( $fam, $directory);

while (1) {
	do {
		$event	= $fam->next_event;
		$dir	= $fam->which($event);
		$file	= $event->filename;

		$filepath = "$dir/$file";
		# Si l on modifie un repertoire $dir == $file
		$filepath = "$dir/" if ($dir eq $file);

		# Si c est un repertoire, on capte seulement la creation 
		if ( -d $filepath ) {
			if ( $event->type =~ /^(create)$/ ) {
				# On enregistre la creation si demand
				print_log ( "Create\tDir  : $filepath" ) if ( $CREATE );
				
				# On ajoute le ou les nouveaux repertoires pour les surveiller aussi
				add_directory_to_fam( $fam, $filepath);
			}
			else {
				# On enregistre la creation d'autre chose qu un repertoire en mode debug
				print_log ( $event->type . "\t$filepath", "debug");
			}
		}
		else {
			if ( $event->type =~ /^(create)$/ ) {
				# On enregistre la creation d'un fichier si demand
				print_log ( "Create\tFile : $filepath" ) if ( $CREATE );
			}
			elsif ( $event->type =~ /^(change)$/ ) {
				# On enregistre le changement d'un fichier si demand
				print_log ( "Change\tFile : $filepath" ) if ( -e $filepath && $CHANGE );
			}
			elsif ( $event->type =~ /^(delete)$/ ) {
				# On enregistre la suppression du fichier ou du repertoire si demand
				print_log ( "Delete\t$filepath" ) if ( $DELETE );
			}
			else {
				# On enregistre le reste en mode debug
				print_log ( $event->type . "\t$filepath", "debug" );
			}
		}

	} while $fam->pending;
}

sub add_directory_to_fam {
	my $fam		= shift;
	my $directory	= shift;
	
	my @directories;
	
	find(sub { -d && -e && push @directories, $File::Find::name; }, $directory);

	for (@directories){
		print_log ( "setting monitor on $_", "debug" );
		$fam->monitor( $_ );
	}
}


## ###################################
## Liste des utilisateurs connects

# list_connections([share])
# Uses the smbstatus program to return a list of connections a share. Each
# element of the returned list is of the form:
#  share, user, group, hostname, date/time
sub list_connections {
	my($started, @rv);

	# New samba status format
	my %pidmap;
	my $line;
	my $out = `$smb_status -s $smb_conf 2>&1`;

	# -s option not supported
	$out = `$smb_status 2>&1` if ( $? );

	foreach my $l (split(/\n/ , $out)) {
		if ($l =~ /^----/) {
			$started++;
		}
		elsif ($started == 1 && $l =~ /^\s*(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+\((\S+)\)/) {
			# A line with a PID, username, group and hostname
			$pidmap{$1} = [ $1, $2, $3, $4, $5 ];
		}
		elsif ($started == 2 && $l =~ /^\s*(\S+)\s+(\d+)\s+(\S+)\s+(.*)$/) {
			# A line with a service, PID and machine. This must be combined data
			# from the first type of line to create the needed information
			my $p = $pidmap{$2};
			
			if (!$_[0] || $1 eq $_[0] || $1 eq $p->[1] && $_[0] eq "homes") {
				push( @rv, [ $1, $p->[1], $p->[2], $3, $4 ]);
			}
		}
	}

	foreach ( @rv ) {
		$line .= "\t" . join("\t", @{ $_ } ) . "\n";
	}

	return $line;
}

# ## ###################################
# ## Recuperation des arguments
# 
# sub getCLOptions {
# 	Getopt::Long::Configure('noignore_case', 'bundling', 'noauto_abbrev');
# 	unless (
# 		GetOptions(
# 			'debug|d'	=> \$DEBUG,
# 			'change'	=> \$CHANGE,
# 			'create'	=> \$CREATE,
# 			'delete'	=> \$DELETE,
# 			'all'		=> \$ALL,
# 			)
# 		)
# 	{	die "abandon : option incorrecte \n"; }
# 
# }

## ###################################
## Affichage

sub print_log {
	my $message	= shift;
	my $niveau	= shift;

	if ( $niveau eq "debug" ) {
		# Niveau debug demande, affiche les messages de debug
		save_log( "\t$message" ) if ( $DEBUG );
	}
	else {
		# Affiche les autres messages que debug
		save_log( "\t$message" );
	}
}

sub save_log {
        open LOG, ">>$log_file" or die print STDERR "ERROR Redirection impossible.\n";

	my $date = strftime("%Y-%m-%d %H:%M:%S",localtime);

        print LOG "$date $_[0]\n";
	print LOG list_connections();
	print LOG "---------------------------------------------------\n";
        close LOG;
}

sub print_error {
	my $message = shift;
	print "Erreur: " . $message . "\n";
	exit;
}
